package de.hdm_stuttgart.mi.sd1.task2;

import java.util.Arrays;

/**
 * <p>String related helper methods.</p>
 */
public class StringHelper {

    /**
     *
     * <p>Restrict an array of arbitrary type objects to a given list of types and converting them to {@link String}
     *  instances.</p>
     *
     * <table class="goikTableDefaults">
     *     <caption>Classes to be filtered</caption>
     *     <tr>
     *         <th colspan="4">Class</th>
     *     </tr>
     *     <tr>
     *         <td>{@link String}</td>
     *         <td>{@link StringBuilder}</td>
     *         <td>{@link StringBuffer}</td>
     *         <td>{@link Boolean}</td>
     *     </tr>
     *     <tr>
     *         <td>{@link Integer}</td>
     *         <td>{@link Long}</td>
     *         <td>{@link Float}</td>
     *         <td>{@link Double}</td>
     *     </tr>
     * </table>
     *
     * <p>Consider the following example list of objects:</p>
     *
     * <pre><code class="java"> Object[] collection = {"Foo", null, new StringBuffer("Eve"), 42, new StringBuilder("Adam"), new Scanner(System.in), 3.7f, true};
     * //            Array index: 0     1                        2   3                          4                       5     6     7</code></pre>
     *
     * <p>By virtue of boxing the <code>collection</code> array references:</p>
     *
     * <table class="goikTableDefaults">
     *     <caption>Objects being contained in variable <code>collection</code></caption>
     *     <tr>
     *         <th>Array index</th>
     *         <th>Class</th>
     *         <th>Array index</th>
     *         <th>Class</th>
     *         <th>Array index</th>
     *         <th>Class</th>
     *         <th>Array index</th>
     *         <th>Class</th>
     *     </tr>
     *     <tr>
     *         <td>0</td>
     *         <td>{@link String}</td>
     *         <td>1</td>
     *         <td><code>null</code> (empty reference)</td>
     *         <td>2</td>
     *         <td>{@link StringBuffer}</td>
     *         <td>3</td>
     *         <td>{@link Integer}</td>
     *     </tr>
     *     <tr>
     *         <td>4</td>
     *         <td>{@link StringBuilder}</td>
     *         <td>5</td>
     *         <td>{@link java.util.Scanner}</td>
     *         <td>6</td>
     *         <td>{@link Float}</td>
     *         <td>7</td>
     *         <td>{@link Boolean}</td>
     *     </tr>
     * </table>
     *
     * <p>Since {@link java.util.Scanner} is not among the to be filtered classes list it will be ignored as
     *     well as the <code>null</code> value. On execution we get:</p>
     *
     * <table class="goikTableDefaults">
     *   <caption>Filtering {@link String} instances</caption>
     *   <tr>
     *     <th>Code</th>
     *     <th>Result</th>
     *   </tr>
     *   <tr>
     *       <td>
     *          <pre><code class="java"> final Object[] collection = {"Foo", null, ...
     *
     * final String[] results = StringHelper.filterStrings(collection);
     *
     * for (final String r: results) {
     *     System.out.println('"' + r + '"');
     * }</code></pre>
     *       </td>
     *       <td>
     *           <pre><code class="nohighlight"> "Foo"
     * "Eve"
     * "42"
     * "Adam"
     * "3.7"
     * "true"</code></pre>
     *       </td>
     *   </tr>
     * </table>
     *
     * @param objects Either <code>null</code> or a list of instances possibly containing <code>null</code> values.
     * @return A filtered list of {@link String} instances.
     */
    static public String[] filterStrings(final Object[] objects) {

        if (null == objects || 0 == objects.length) {
            return new String[0];
        } else {
            final String[] retrievedStrings = new String[objects.length];
            int count = 0;
            for (final Object o: objects) {
                if (o instanceof String ||
                    o instanceof StringBuilder ||
                        o instanceof StringBuffer ||
                        o instanceof Boolean ||
                    o instanceof Number) {
                    retrievedStrings[count++] =  o.toString();
                }
            }
            if (0 == count) {
                return new String[0];
            } else {
                return Arrays.copyOf(retrievedStrings, count);
            }
        }
    }

    private StringHelper(){ /* Suppressing constructor Javadoc HTML generation */}
}
