package de.hdm_stuttgart.mi.sd1.task2;

/**
 * <p>Matching wind forecast conditions</p>
 */
public class WindForecast {

    private final Direction direction;

    private final int windSpeed;

    /**
     * <p>Wind forecast by range and directions.</p>
     *
     * @param windSpeed Expected wind speed in Km/h.
     * @param directions The main wid direction to be expected
     */
    public WindForecast(final int windSpeed, final Direction directions) {
        this.direction = directions;
        this.windSpeed = windSpeed;
    }

    /**
     * <p>Check for matching conditions.</p>
     *
     * <p>Many water sports addicts depend on suitable wind conditions being described by wind speed and
     * optionally wind direction.</p>
     *
     * <p>As an example a kitesurfer might want to get notified if the wind forecast at his or her suitable spot predicts
     * wind ranging from 30 Km/h to 50 Km/h in speed and coming either from northern or northwestern direction. The
     * wind prediction is wind at 40 Km from north. Corresponding code sample:</p>
     *
     * <pre><code class="java"> final WindForecast forecast = new WindForecast(40, Direction.N);
     *
     * if (forecast.matchingConditions(30, 50, new Direction[]{Direction.N, Direction.NW})) {
     *    System.out.println("Yeah, let's go for it! \uD83D\uDE0E");
     * else {
     *    System.out.println("I hate waiting for the wind! \uD83E\uDD2C");
     * }</code></pre>
     *
     * @param speedMinimum Minimum required wind speed.
     * @param speedMaximum Maximum tolerable wind speed.
     * @param directions List of favourable wind directions
     * @return <code>true</code> if current wind's speed and one of the provided directions match the
     * {@link #WindForecast(int, Direction)} criteria, <code>false</code> otherwise.
     */
    public boolean matchingConditions(final int speedMinimum, final int speedMaximum, final Direction[] directions) {

        if (windSpeed < speedMinimum || speedMaximum < windSpeed) {
            return false;
        }

        if (0 == directions.length) { // Direction does not matter to customer
            return true;
        } else {
            for (final Direction d: directions) {
                if (d == direction) {
                    return true;
                }
            }
            return false;
        }
    }
}
